# FLASCHEL-SEMMLER MODEL
# WITH ADJUSTMENT TO STABILIZE THE OSCILLATIONS
library(colorspace)
library(scales)
library(RColorBrewer)
timelength<-500
Kmax<-3

# INITIALIZATION
labels<-c('M','k','p','r','X','S','D','pi','K','g')
K<-3
capital<-array(0,dim=c(Kmax,length(labels),timelength),dimnames=list(paste0('K',c(1:Kmax)),labels,paste0('t',c(1:timelength))))
dx<-c(0.75,0.75)
dp<-c(0.05,0.05)
# capital[,'M',1]<-capital[2:3,'p',1] %*% At * capital[,'X',1]
# r<-t(t(p)) %*% At %*% t(t(capital[,'X',1]))
capital[,c('r','pi','g'),1]<-NA

# Case 1: Material-saving innovations: square matrix A0 > rectangular matrix At > square matrix Af
At<-matrix(c(0.4,0.3,0.2,0.15,0.6,0.5),nrow=2)
A0<-At[,c(1,3)]
R0<-1/eigen(A0)$values[1] # 1.139
Af<-At
Af<-At[,c(2,3)]
Rf<-1/eigen(Af)$values[1] # 1.459 also for Rt
K<-2
x_0<-c(0.45,0.4)
p_0<-c(0.6,0.6)
capital[1:K,'X',1]<-x_0
capital[1:K,'p',1]<-p_0
capital[1:K,'k',1]<-1:K
C<-diag(2)-A0/eigen(A0)$values[1]
gam<-0.6
K<-2

# We use Rf as constant
# Case 2: Substitution effects: rectangular matrix At > rectangular matrix Att after 70 iterations
# At<-matrix(c(0.4,0.3,0.2,0.15,0.6,0.5),nrow=2)
# Att<-matrix(c(0.15,0.1,0.2,0.15,0.6,0.5),nrow=2)
# C<-cbind(c(1,0),diag(2))-At/eigen(At[,c(2,3)])$values[1]
# x_0<-c(0.45,0.4,0.4)
# p_0<-c(0.6,0.6,0.6)
# K<-3
# capital[1:K,'X',1]<-x_0
# capital[1:K,'p',1]<-p_0
# gam<-0.9

# # Case 3: Joint Production (includes outputs B). Augmented rectangular matrices A,B replace square matrices
# B0<-matrix(c(1,0.1667,0.65,1),nrow=2)
# Bt<-cbind(c(1,0.07),B0)
# A00<-matrix(c(0.9167,0.1389,0.0417,0.9027),nrow=2)
# x_0<-t(t(eigen(A00)$vectors[,1]))
# p_0<-(t(eigen(t(A00))$vectors[,1]))
# At3<-cbind(c(0.98,0.25),A00)
# R3<-1.162
# C<-B0-R3*A00
# x_0<-c(0.45,0.4,0.4)
# p_0<-c(0.6,0.6,0.6)
# K<-3
# capital[1:K,'X',1]<-x_0
# capital[1:K,'p',1]<-p_0
# gam<-5

for (t in 1:(timelength-1))
{
  # Case 1
  if(t==50)
  {
    C<-cbind(c(1,0),diag(K))-At/eigen(Af)$values[1]
    capital[K+1,c('X','p'),t]<-capital[K,c('X','p'),t]
    capital[K,c('X','p'),t]<-c(0.1,0.4)
    K<-K+1
  }
  if(t==100)
  {
    capital[1:(K-1),c('X','p'),t]<-capital[2:K,c('X','p'),t]
    K<-K-1
    C<-diag(K)-Af/eigen(Af)$values[1]
  }
  # if(t==30){C<-I0-Att/eigen(At[,c(2,3)])$values[1]} # Case 2
  # if(t==200) {C<-I0-Att/eigen(Att[,c(1,3)])$values[1]} # Case 2
  # if(t<100){C<-B0-R3*A00} else {C<-Bt-R3*At3} # Case 3
  
  if(ncol(C)==nrow(C))
  {
    X<-t(t(capital[1:K,'X',t]))
    P<-capital[1:K,'p',t]
    capital[1:K,'p',t+1]<-P-dp*t(C %*% X)*P
    # s<-t(C) %*% t(t(capital[1:K,'p',t+1]-P))
     s<--t(C) %*% diag(dp) %*% diag(P) %*% C
    # capital[1:K,'X',t+1]<-X+dx*(t(C) %*% P + gam*s)*X
    capital[1:K,'X',t+1]<-X+dx*X*(t(C) %*% t(t(P)) + gam*s %*% X)
  }
  else
  {
    X<-t(t(capital[1:K,'X',t]))
    P<-capital[c(2,3),'p',t]
    capital[1:K,'p',t+1]<-(P-dp*P*(C %*% X))[c(1,1,2)]
    s<--t(C) %*% diag(dp) %*% diag(P) %*% C # Square Matrix of dimension 3
    capital[1:K,'X',t+1]<-X+c(dx[1],dx[1],dx[2])*X*(t(C) %*% t(t(P)) + gam*s %*% X)
  }
}

colores<-rainbow_hcl(Kmax)
lab_<-c('capital stock (M)',"output (M')",'price (p)','output (X)','profit rate (r)', 'Growth')
names(lab_)<-c('M','pi','p','X','r','K')
tval<-1:300
#tval<-1:timelength
par(mfrow=c(2,3),mar=c(2.5, 3.5, 1.4, 1.1),oma=c(0,0,2.5,0))
for(lab in names(lab_))
{
  yli<-c(min(Re(capital[,lab,tval]),na.rm=T),max(Re(capital[,lab,tval]),na.rm=T))
  # yli<-c(min(avg[,lab,tval],na.rm=T),max(avg[,lab,tval],na.rm=T))
  if(lab=='x'){yli[1]<-0}
  #if(lab=='p'){yli[1]<-4}
  plot(NA,xlim=c(min(tval),max(tval)),ylim=yli,xlab='',ylab='',main=lab_[lab],font.lab=2,cex.lab=1)
  grid()
  #if(lab=='r' | lab=='p'){for(i in 1:N){points(tval,capital[i,lab,tval],col=alpha(colori[N-i+1],0.25),cex=0.8,pch=16)}} else{for(i in 1:N){lines(tval,capital[i,lab,tval],col=alpha(colori[N-i+1],0.25),lwd=2)}}
  for(k in 1:Kmax){lines(tval,capital[k,lab,tval],col=colores[k],lwd=2.25)}
  # if(lab=='x'){for(k in 1:K){lines(tval,yli[2]*N_h[tval,k]/N,col=rgb(t(col2rgb(colores[k])/1.4),maxColorValue=255),lwd=1.9,lty=3)}}
  # if(lab=='x'){axis(4,at=seq(0,yli[2],length.out=5),labels=seq(0,yli[2],length.out=5)*N/yli[2])}
  #if(lab=='p'){for(k in 1:K){lines(tval,p0[k,tval],col='black',lwd=0.8,lty=2)}}
  if(lab=='p'){abline(h=p0,col='black',lwd=0.8,lty=2)}
  if(lab=='p'){mtext(paste0('p',1:K),side=4,at=p0,las=1,cex=0.8,font=2)}
  if(lab=='X'){abline(h=x0,col='black',lwd=0.8,lty=2)}
  if(lab=='X'){mtext(paste0('X',1:K),side=4,at=x0,las=1,cex=0.8,font=2)}
  if(lab=='r'){abline(h=r_eq,lty=2)}
  legend('topleft',paste0('K',1:K),col=colores,pch=16,pt.cex=2,cex=0.9,bty='n')
  mtext('time',side=1,cex=0.8,line=2,font=2)
}