library(reshape2)
library(dplyr)
library(ggplot2)
library(ggnewscale)
library(scales)


#### PLOT SINGLE SIMULATION (SIMULATION)
df <- simulation$long
levels(df$variable) <- c('relative price (p)','gross output (X)','profit rate (r)', 'growth (g)')
params <- unlist(simulation$par)
sectors <- grep('C19', levels(df$industry))
df$industry <- factor(df$industry, levels = c(levels(df$industry)[-sectors], levels(df$industry)[sectors]))

alpha.values <- c(rep(0.5, times = length(levels(df$industry)) - 2), 1, 1)
size.values <- c(rep(0.5, times = length(levels(df$industry)) - 2), 1.2, 1.2)
col.values <- c(hue_pal()(length(levels(df$industry)) - 2), hue_pal()(2))

ggplot(df) + facet_wrap(~ variable, scales = 'free') + ylab('') +
  labs(title = 'Simulation of the Low-Carbon Transition', 
       subtitle = bquote(theta == .(params['theta']) ~~ sigma[0] == .(params['q_0']) ~~ tau == .(params['tax']) ~~ rho == .(params['subsidy_rate']))) +
  theme(plot.title = element_text(size = 14, face = 'bold')) +
  geom_line(data = df, aes(x = time, y = value, colour = industry, alpha = industry, size = industry), inherit.aes = F, show.legend = T) +
  scale_alpha_manual(breaks = levels(df$industry), values = alpha.values) +
  scale_size_manual(breaks = levels(df$industry), values = size.values) +
  scale_color_manual(breaks = levels(df$industry), values = col.values)



##### PLOT MANY SIMULATIONS (UNIVERSE)
plot.universe <- function(universe, title)
{
longs <- lapply(universe, function (x) x$long)
# names(longs) <- countries[1:5]
params <- sapply(universe, function (x) x$par)
# sec_tor <- unique(sapply(universe, function (x) x$sector))
params <- unlist(apply(params, 1, function (x) unique(x)[1]))
durations <- data.frame(country = names(sapply(universe, function (x) x$duration)), 
                        value = sapply(universe, function (x) x$duration))

df <- bind_rows(longs, .id = 'country')
levels(df$variable) <- c('relative price (p)','gross output (X)','profitability (r)', 'growth (g)')
sectors <- grep('Process', levels(df$industry))
df$industry <- factor(df$industry, levels = c(levels(df$industry)[-sectors], rev(levels(df$industry)[sectors])))

alpha.values <- c(rep(0.5, times = length(levels(df$industry)) - 2), 1, 1)
size.values <- c(rep(0.5, times = length(levels(df$industry)) - 2), 1.2, 1.2)
col.values <- c(hue_pal()(length(levels(df$industry)) - 2), hue_pal()(2))

# pdf('scenario_only_tax.pdf', width = 12, height = 7.5)
ggplot() + facet_grid(variable ~ country, scales = "free_y") + ylab('') +
  labs(title = title, 
       subtitle = bquote(theta == .(params['theta']) ~~ sigma[0] == .(params['q_0']) ~~ tau == .(params['tax']) ~~ rho == .(params['subsidy_rate']))) +
  theme(plot.title = element_text(size = 14, face = 'bold')) +
  geom_line(data = df, aes(x = time, y = value, colour = industry, alpha = industry, size = industry), inherit.aes = F, show.legend = T) +
  scale_alpha_manual(breaks = levels(df$industry), values = alpha.values) +
  scale_size_manual(breaks = levels(df$industry), values = size.values) +
  scale_color_manual(breaks = levels(df$industry), values = col.values) +
  geom_vline(data = durations, aes(xintercept = value), linetype = 'dashed', col = 'darkgray') +
  theme(axis.text.x = element_text(size = 7.5)) # theme(axis.text.x = element_text(angle = 30, hjust = 1))
}
# 



########
### OLD BASE PLOT
library(colorspace)
colores <- rainbow_hcl(K)
lab_ <- c('relative price (p)','gross output (X)','profit rate (r)', 'growth (g)')
names(lab_) <- c('p','X','r','g')
# tval <- 1:timelength
tval <- 1:100

par(mfrow = c(2,2), mar = c(3, 2.7, 1.6, 1.4), oma = c(0,0,0.75,0))

for(la0 in names(lab_))
{
  yli <- c(min(Re(capital[1:max(Nt),la0,tval]),na.rm=T),max(Re(capital[1:max(Nt),la0,tval]),na.rm=T))
  if(la0 == 'g') {yli[1]  <-  min(Re(capital[1:max(Nt),la0,(TIO + 1):max(tval)]),na.rm=T)}
  # yli <- c(min(avg[,lab,tval],na.rm=T),max(avg[,lab,tval],na.rm=T))
  # if(la0=='g' & subsidy){yli[2] <- 0.05*yli[2]}
  plot(NA,xlim=c(min(tval),max(tval)*(1+.05)),las=1,ylim=yli,xlab='',ylab='',main=lab_[la0],font.lab=2,cex.lab=1)
  grid()
  #if(lab=='r' | lab=='p'){for(i in 1 : N){points(tval,capital[i,lab,tval],col=alpha(colori[N-i+1],0.25),cex=0.8,pch=16)}} else{for(i in 1 : N){lines(tval,capital[i,lab,tval],col=alpha(colori[N-i+1],0.25),lwd=2)}}
  if(innovation)
  {
    for(k in 1:max(Nt))
    {
      lines(min(tval):(TIO-1),capital[k,la0,min(tval):(TIO-1)],col=alpha(colores[capital[k,'k',1]],0.5),lwd=2.25)
      lines(TIO:max(tval),capital[k,la0,TIO:max(tval)],col=alpha(colores[capital[k,'k',TIO]],0.5),lwd=2.25)
    }
    text(c(TIO+15,duration+TIO+15),rep(0.95*yli[2],2),c('start','end'),col=gray(0.3),cex=0.8)
    abline(v= c(TIO,TIO-1),lty=2)
    abline(v= duration + TIO,lty=2,col='darkolivegreen4',lwd=2)
    lines(min(tval):(TIO-1),capital[sectors[1],la0,min(tval):(TIO-1)],col='tomato',lwd=2.25)
    lines((TIO+1):max(tval),capital[sectors[2],la0,(TIO+1):max(tval)],col='steelblue',lwd=2.25)
    lines((TIO+1):max(tval),capital[sectors[1],la0,(TIO+1):max(tval)],col='tomato',lwd=2.25)
    legend('topleft',c('carbon',"green"),col=c('tomato','steelblue'),pch=16,bty='n')
  }
  else {for(k in 1:max(Nt)) {lines(tval,capital[k,la0,tval],col=alpha(colores[capital[k,'k',1]],0.5),lwd=2.25)}}
  
  text(max(tval)*(1+.05),capital[1:max(Nt),la0,max(tval)],labels=seko[capital[1:max(Nt),'k',max(tval)]],col=colores)
  if(innovation) {text(max(tval)*(1+.05),capital[sectors,la0,max(tval)],labels=paste(thesector,c('','*'),sep=''),col=c('tomato','steelblue'),font=2)}
  # if(la0=='p'){abline(h=p_0,lwd=0.8,lty=2,col=alpha(colores,0.75))}
  # if(la0=='X'){abline(h=x0,lwd=0.8,lty=2,col=alpha(colores,0.75))}
  if(la0=='r')  {lines(tval,Rt[tval],lty=3,col=alpha('black',0.75))}
  if(la0=='r')  {abline(h=mean(Rt[tval]),lty=2,col=alpha('black',0.75))}
  if(la0=='g'){abline(h=0,lty=2)}
  mtext('time',side=1,cex=0.8,line=2,font=2)
}
mtext(zou,side=3,line=-.75,outer=T,cex=1.5,font=2)
print(duration)






# new_scale_colour() +
# geom_line(data = df[df$process %in% c('carbon', 'green'),], aes(x = time, y = value, colour = process), inherit.aes = F, show.legend = T)
ggplot(df, mapping = aes(x = time, y = value, alpha = process)) + 
  geom_path(data = df[df$process %in% c('carbon', 'green'),], aes(x = time, y = value, colour = process), inherit.aes = F, show.legend = T) + 
  new_scale_colour() + 
  geom_line(data = df, aes(x = time, y = value, colour = industry, alpha = alpha), inherit.aes = F, show.legend = T) +
  #   guides(fill = guide_legend(override.aes = list(colour=NA))) + 
  new_scale_colour() + 
  + 
  facet_grid(variable ~ country, scales = "free_y") + ylab('') +
  labs(title = 'Simulation Scenarios of the Low-Carbon Transition', 
       subtitle = bquote(theta == .(params['theta']) ~ sigma[0] == .(params['q_0']) ~ tau == .(params[2]) ~ rho == .(params['subsidy_rate']))) +
  theme(plot.title = element_text(size = 14, face = 'bold'))

pdf('test.pdf')
ggplot(df, mapping = aes(x = time, y = value)) + 
  geom_line(data = df[df$process %in% c('carbon', 'green'),], aes(x = time, y = value, colour = process), inherit.aes = F, show.legend = T) + 
  scale_color_hue() +
  new_scale_colour() + 
  geom_line(data = df, aes(x = time, y = value, colour = industry), inherit.aes = F, show.legend = T) +
  scale_color_hue() +
  new_scale_colour() + 
  geom_path(data = df[df$process %in% c('carbon', 'green'),], aes(x = time, y = value, colour = process), inherit.aes = F, show.legend = F) + 
  facet_grid(variable ~ country, scales = "free_y") + ylab('') +
  labs(title = 'Simulation Scenarios of the Low-Carbon Transition', 
       subtitle = bquote(theta == .(params['theta']) ~ sigma[0] == .(params['q_0']) ~ tau == .(params[2]) ~ rho == .(params['subsidy_rate']))) +
  theme(plot.title = element_text(size = 14, face = 'bold'))
dev.off()




# geom_line(data = df[df$process %in% c('carbon', 'green'),], aes(x = time, y = value, colour = process), inherit.aes = F, show.legend = T) + 
# scale_color_hue() +
# new_scale_colour() + 

new_scale_colour() + 
  geom_path(data = df[df$process %in% c('carbon', 'green'),], aes(x = time, y = value, colour = process), inherit.aes = F, show.legend = F) + 
  scale_color_hue() +
