cof.pos <- coefz.p[, c('B','C19','D')] > 0 & coefz.x[, c('B','C19','D')] > 0
table(coefz.p > 0)
table(coefz.x > 0)
cof.pos

##
library(ggplot2)
library(scales)
library(dplyr)

param <- list('No Policy' = list(timelength = 500,
                                  q_0 = 0.25,
                                  tax = 0,
                                  theta = .7,
                                  subsidy_rate = 0,
                                  gam = 0),
               'Only Carbon Pricing' = list(timelength = 500,
                                  q_0 = 0.055,
                                  tax = 0,
                                  theta = .1,
                                  subsidy_rate = 0,
                                  gam = 0),
               'Only Percentage Tax' = list(timelength = 150,
                                  q_0 = 0.055,
                                  tax = 0.025,
                                  theta = .9,
                                  subsidy_rate = 0,
                                  gam = 0),
               'Tax-Subsidy Mix' = list(timelength = 150,
                                          q_0 = 0.055,
                                          tax = 0.025,
                                          theta = .9,
                                          subsidy_rate = 0.5,
                                          gam = 0))
scenarios <- names(param)
duration.table <- list()
uni.scene <- list()

for(scenario in scenarios)
{
  parameters <- param[[scenario]]
  uni.scene[[scenario]] <- list('DEU.C19' = simulate(parameters, 'DEU', 'C19'),
                                'FRA.C19' = simulate(parameters, 'FRA', 'C19'),
                                'ITA.C19' = simulate(parameters, 'ITA', 'C19'),
                                'NLD.C19' = simulate(parameters, 'NLD', 'C19'),
                                'DEU.B' = simulate(parameters, 'DEU', 'B'),
                                'ITA.B' = simulate(parameters, 'ITA', 'B'),
                                'JPN.D' = simulate(parameters, 'JPN', 'D'))
  
  duration.table[[scenario]] <- sapply(uni.scene[[scenario]], function (x) x$duration)
}

for (i in 1 : length(uni.scene))
{
  scenario <- scenarios[[i]]
  name <- gsub(' ','_', scenario)
  name <- gsub('-','_', name)
  pdf(paste0('fig', i + 4, '_', name,'.pdf'), width = 12, height = 7.5)
  print(plot.universe(universe = uni.scene[[scenario]], 
              title = paste0('Scenario Simulation of the Low-Carbon Transition: ', scenario)))
  dev.off()
}

scenario <- scenarios[[1]]
pdf(paste0(gsub(' ','_', scenario),'.pdf'), width = 12, height = 7.5)
plot.universe(universe = uni.scene[[scenario]], 
              title = paste0('Scenario Simulation of the Low-Carbon Transition: ', scenario))
dev.off()
scenario <- scenarios[[2]]
pdf(paste0(gsub(' ','_', scenario),'.pdf'), width = 12, height = 7.5)
plot.universe(universe = uni.scene[[scenario]], 
              title = paste0('Scenario Simulation of the Low-Carbon Transition: ', scenario))
dev.off()
scenario <- scenarios[[3]]
pdf(paste0(gsub(' ','_', scenario),'.pdf'), width = 12, height = 7.5)
plot.universe(universe = uni.scene[[scenario]], 
              title = paste0('Scenario Simulation of the Low-Carbon Transition: ', scenario))
dev.off()
scenario <- scenarios[[4]]
pdf(paste0(gsub(' ','_', scenario),'.pdf'), width = 12, height = 7.5)
plot.universe(universe = uni.scene[[scenario]], 
              title = paste0('Scenario Simulation of the Low-Carbon Transition: ', scenario))
dev.off()
scenario <- scenarios[[5]]
pdf(paste0(gsub(' ','_', scenario),'.pdf'), width = 12, height = 7.5)
plot.universe(universe = uni.scene[[scenario]], 
              title = paste0('Scenario Simulation of the Low-Carbon Transition: ', scenario))
dev.off()

paris <- data.frame(lab = factor(paste(c(16,23,51,65),'years')), value = c(16,23,51,65), size = 1.5*c(16,23,51,65)/65)


df <- melt(as.matrix(as.data.frame(duration.table)), varnames = c('target.industry','scenario'))
levels(df$scenario) <- gsub('\\.',' ', levels(df$scenario))
pdf('fig10_decarb_time.pdf', width = 10, height = 6)
ggplot() + 
  geom_bar(data = df, aes(x = scenario, y = value, fill = target.industry), stat = 'identity', position="dodge") +
  geom_hline(data = paris, aes(yintercept = value), linetype = 'dashed', alpha = .6) + 
  # scale_x_discrete(limits = df$scenario) + scale_y_continuous(limits = c(0,500)) +
  xlab('') + ylab('years') + ggtitle('Decarbonization Time for Selected Policy Scenarios') +
  geom_text(data = paris, show.legend = F, alpha = .75, 
            fontface = 'bold', size = 2.5, vjust = -.5, hjust = 0,
            aes(x = .5, y = value, label = lab)) +
  annotate("text", x = 5.7 , y = mean(paris$value), angle = 270, label="UN IPCC Targets", size = 3, fontface = 'bold') + 
  coord_cartesian(ylim = c(0, 500), xlim = c(1,5), clip = "off")
dev.off()


################ 
library(broom)
library(stargazer)

phaselong <- readRDS(file="phaselong_10_total.RDS")
df <- phaselong
df$speed <- 1/df$value
model <- lm(speed ~ q0 + eff + tax + subsidy, data = df)
stargazer(model, type = 'latex', flip = T)

foo <- t(as.data.frame(tidy(model)))


